<?php

App::uses('BuzzBookingsAppController', 'BuzzBookings.Controller');

class BookingsController extends BuzzBookingsAppController {

	public function beforeFilter() {
		parent::beforeFilter();

		$this->Auth->allow(array(
			'index',
			'packages',
			'date',
			'extras',
			'ajax_voucher',
			'ajax_date'
		));
	}

/**
 * Bookings landing page - will mostly get bypassed by homepage/activity pages
 *
 * @return void
 */
	public function index() {
		$Booking = $this->{$this->modelClass};

		if (!empty($this->request->data)) {

			if (!empty($this->request->data['Booking']['activity_id'])) {
				// Forward to the correct packages step.
				return $this->redirect([
					'action' => 'packages',
					$this->request->data['Booking']['activity_id']
				]);
			} else {
				$this->Session->setFlash(
					__d('BuzzBookings', 'Please select an activity'),
					'flash_fail'
				);
			}

		}

		$activities = $Booking->Activity->getListed();
		$activities = Hash::combine($activities, '{n}.Activity.id', '{n}.Activity.name');

		if (count($activities) === 1) {
			return $this->redirect([
				'action' => 'packages'
			]);
		} elseif (empty($activities)) {
			// No activities have been setup or created so throw an exception
			throw new NotFoundException();
		}

		$this->set(compact('activities'));
		$this->view = 'BuzzBookings./Bookings/index';

		return;
	}

/**
 * Packages step - add packages and vouchers to a booking
 *
 * To jump to the packages step for a specific activity the activity ID can be
 * passed; otherwise the activity ID is set on the booking.
 *
 * To preselect a specific package a package ID needs to be passed.
 *
 * @param int $activityId Activity ID
 * @param int $packageId Optional activity package ID
 * @return void
 */
	public function packages($activityId = null, $packageId = null) {
		$Booking = $this->{$this->modelClass};

		// Process form data.
		if (!empty($this->request->data)) {

			$result = $Booking->createBasket($activityId, $this->request->data['BookingItem']);
			if ($result === true) {
				return $this->redirect(['action' => 'date']);
			} else {
				$this->Session->setFlash(
					__d('BuzzBookings', 'Please select a package'),
					'flash_fail'
				);
			}

		}

		// Get the basket.
		$basket = $Booking->getBasket();

		if (empty($activityId)) {

			if (!empty($basket['Booking']['activity_id'])) {
				// Use the basket's activity.
				$activityId = $basket['Booking']['activity_id'];
			} else {
				// Check if the site is configured to only have one listed
				// bookable activity. This is needed to prevent a redirect
				// loop with this and the previous step.
				$activityId = $Booking->Activity->onlyBookableActivity();
			}

		}

		// If the activity ID hasn't been set then redirect to the previous
		// step.
		if (empty($activityId)) {
			return $this->redirect([
				'action' => 'index'
			]);
		} else {
			// Set the booking's activity ID.
			$this->request->data['Booking']['activity_id'] = $activityId;
		}

		// Get the packages and activity details for generating the form.
		$data = $Booking->Activity->getBookableActivity($activityId);

		if (empty($data)) {
			throw new NotFoundException();
		}

		if (
			!empty($basket) && (int)$basket['Booking']['activity_id'] !== (int)$activityId
		) {
			// Clear the basket as we're either changing the activity being booked so
			// existing items are no longer valid.
			$Booking->clearBasket(false);
			// We don't need to pass the basket to the view anymore as we're
			// essentially starting a fresh basket.
			$basket = [];
		} elseif (empty($this->request->data['BookingItem']) && !empty($basket['BookingItem'])) {
			// Populate the form with existing booking items.
			$this->request->data['BookingItem'] = Hash::combine($basket['BookingItem'], '{n}.activity_package_id', '{n}');
		}

		// If someone's followed a link to directly book a specific package
		// then set the quantity to 1.
		if (!empty($packageId) && empty($this->request->data['BookingItem'])) {
			$this->request->data['BookingItem'][$packageId]['quantity'] = 1;
		}

		$previousStep = ['action' => 'index'];
		// Check if this is a non-listed activity (i.e. we will need to return
		// the user to the activity page rather than allow them to select a
		// different activity).
		if (!empty($basket['Activity']['listed']) && (bool)$basket['Activity']['listed'] === false) {
			$previousStep = ['controller' => 'activities', 'action' => 'view', $activityId];
		}

		$this->set(compact('data', 'basket', 'previousStep'));
		$this->view = 'BuzzBookings./Bookings/packages';

		return;
	}

/**
 * Date step - book a time and date for the booking
 *
 * @return void
 */
	public function date() {
		$Booking = $this->{$this->modelClass};

		// Get the basket.
		$basket = $Booking->getBasket();
		if (empty($basket)) {
			$this->Session->setFlash(
				__d('BuzzBookings', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect(['action' => 'index']);
		}

		$previousStep = ['action' => 'packages', $basket['Booking']['activity_id']];

		$this->set(compact('basket', 'previousStep'));
		$this->view = 'BuzzBookings./Bookings/date';

		return;
	}

	public function extras() {
		$Booking = $this->{$this->modelClass};

		// Get the basket.
		$basket = $Booking->getBasket();
		if (empty($basket)) {
			$this->Session->setFlash(
				__d('BuzzBookings', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect(['action' => 'index']);
		}

		$extras = $Booking->getExtras($basket);

		$previousStep = ['action' => 'date'];

		$this->set(compact('basket', 'extras', 'previousStep'));
		$this->view = 'BuzzBookings./Bookings/extras';

		return;
	}

/**
 * AJAX call for generating voucher fields.
 *
 * @param int $activityId Activity ID
 * @return void
 */
	public function ajax_voucher($activityId) {
		$Booking = $this->{$this->modelClass};

		$data = $Booking->getVouchers($activityId);

		// Build a simple array of voucher providers for convenience in the View.
		$voucherProviders = Hash::combine($data, '{n}.VoucherProvider.id', '{n}.VoucherProvider.name');
		asort($voucherProviders);

		$this->set(compact('data', 'voucherProviders'));
		$this->layout = 'ajax';
		$this->view = 'BuzzBookings./Bookings/ajax_voucher';

		return;
	}

/**
 * AJAX call for generating the available times and dates for the current
 * booking.
 *
 * @return void
 */
	public function ajax_date() {
		$Booking = $this->{$this->modelClass};

		// Get the available times and dates.
		$fromDate = date('Y-m-d');
		$toDate = date('Y-m-d', strtotime('+3 months'));
		$data = $Booking->getAvailableDates($fromDate, $toDate);

		// Work out the unavailable dates within our date range.
		$unavailableDates = $Booking->calculateUnavailableDates($data, $fromDate, $toDate);

		$this->set(compact('data', 'unavailableDates'));
		$this->layout = 'ajax';
		$this->view = 'BuzzBookings./Bookings/ajax_date';

		return;
	}

}
